%
%   This function transforms the interpolation data for the
%    multiple Nevanlinna-Pick interpolation problem subjected
%    to the bilinear transformation
%
%               af(z) + b                      b - dg(z)
%       g(z) = -----------     <-->    f(z) = ----------- 
%               cf(z) + d                      cg(z) - a
%
%   Input:  a,b,c,d  -  Transformation parameters
%           w        -  Interpolation data for f 
%           where  w = [ w0 w1 ... wm-1 ]
%
%   Output: v  -  Interpolation data for g
%           where  v = [ v0 v1 ... vm-1 ]
%
%             f(z) = w0 
%            f'(z) = w1 
%                  .      
%                  .       
%                  .       
%       f(m-1)(z)
%      ----------- = w(m-1)
%        (m-1)!
%
%
%             g(z) = v0 
%            g'(z) = v1 
%                  .      
%                  .       
%                  .       
%       g(m-1)(z)
%      ----------- = v(m-1)
%        (m-1)!
%
function [ V ] = BilinearRangeDataLow(a,b,c,d,W)

% Determine interpolation point and function value
R = inv(c*W{1} + d*eye(size(W{1},1)));
V{1} = (a*W{1} + b*eye(size(W{1},1))) * R;

% Determine multiplicity
mult = size(W,2);

if(mult >= 2)
    % Determine first derivative interpolation data
    V{2} = (a*W{2} - c*V{1}*W{2}) * R;
    
    % Determine higher order derivative interpolation data (if any...)
    coeff = [ 1 1 ];
    fact = [ 1 1 ];
    for k = 3:mult
        coeff = conv(coeff,[1 1]);
        fact = [ factorial(k-1) fact ];
        V{k} = a*W{k}*fact(1);
        for l = 1:k-1
            V{k} = V{k} - c*V{l}*fact(end-l+1)*W{k-l+1}*fact(l)*coeff(l);
        end
        V{k} = V{k} * R / fact(1);
    end
end