%
%   This is a main script for the design example called "AIRC model" from [1]
%    using the procedure of [2]. The plant can be found in appendix A1 of [1].
%    Furthermore, several techniques are used through out the book in order
%    to deal with the design example.
%
%	Specification of system. Define the transfer matrix for the corresponding
%    "Standard Problem" of "A Course in Hinf Control" by B.A. Francis:
%
%                   z              w
%         <-------------|-----|<-------------
%                       |  G  |
%               |-------|-----|<-------|
%               |                      |
%           y   |                      | u
%               |       |-----|        |
%               |------>|  K  |--------|
%                       |-----|
%
%           xdot =  A * x +  B1 * w +  B2 * u
%			z    = C1 * x + D11 * w + D12 * u
%			y    = C2 * x + D21 * w + D22 * u
%
%     x1 = altitude,      x2 = forward speed,         x3 = pitch angle, 
%                         x4 = pitch rate,            x5 = vertical speed
%     u1 = spoiler angle, u2 = forward accerleration, u3 = elevator angle
%     y1 = altitude,      y2 = forward speed,         y3 = pitch angle 
%
%   1. J.M. Maciejowski, "Multivariable Feedback Design", Addison-Wesley, 1989
%
%   2. A. Blomqvist, A. Lindquist & R. Nagamune, "Matrix-valued 
%       Nevanlinna-Pick interpolation with complexity constraint: an
%       optimization approach", preprint 2002.
%
clear all, format compact

%
%   Load the model and Hinf design
%
load AIRCmodel
load AIRCHinfdesign

%
%   Perform NPDC design
%
indata.addpoints = [ 1e8 ];
indata.gamma = 10^(12/20);
indata.kappa = 0.97;
indata.speczeros = [18i 12]; %RHP
indata.maxdist = 0.80;
indata.lambda = 1;
indata.poleposition = 1e-6;

low = 0;high = 2;
tvec = 0:.01:3;
xvec = logspace(low,high,1000);
axvec = [10^low 10^high -30 10];
figure(1), clf, hold on
figure(2), clf, hold on
figure(3), clf, hold on

vec = { {[10^(20/20)]}};

for x = vec

   indata.gamma = x{1}{1}; 
   outdata = ExAIRCNPDC(indata);

end

%
%   Plot the frequency response
%
[svSNPDC,xvec] = sigma(outdata.clsysNPDC,'k-',xvec);
[svTNPDC,xvec] = sigma(eye(3)-outdata.clsysNPDC,'k-',xvec);
[svSHInf,xvec] = sigma(ltisys2sys(clsysHinf),xvec);
[svTHInf,xvec] = sigma(eye(3)-ltisys2sys(clsysHinf),xvec);

figure(1), clf
subplot(1,2,1)
semilogx(xvec,20*log10(svSNPDC(1,:)),'k-'), hold on
semilogx(xvec,20*log10(svSHInf(1,:)),'k--')
semilogx(xvec,20*log10(svSNPDC(2:end,:)),'k-')
semilogx(xvec,20*log10(svSHInf(2:end,:)),'k--')
%title('Sensitivity functions, S')
xlabel('Frequency (rad/s)')
ylabel('Singular values (dB)')
legend('Our design','Optimal H^\infty design',4)

subplot(1,2,2)
semilogx(xvec,20*log10(svTNPDC(1,:)),'k-'), hold on
semilogx(xvec,20*log10(svTHInf(1,:)),'k--')
semilogx(xvec,20*log10(svTNPDC(2:end,:)),'k-')
semilogx(xvec,20*log10(svTHInf(2:end,:)),'k--')
%title('Complementary sensitivity functions, T')
xlabel('Frequency (rad/s)')
ylabel('Singular values (dB)')
legend('Our design','Optimal H^\infty design',3)

%
%   Plot the step responses
%
figure(2),clf, hold on
PC = minreal(sys*outdata.sysKNPDC); 
stepsysNPDC = minreal(inv(eye(3)+PC)*PC);
PC = minreal(sys*ltisys2sys(KHinf)); 
stepsysHinf = minreal(inv(eye(3)+PC)*PC);
for k = 1:3
    for l = 1:3
        [y,t] = step(tvec,stepsysNPDC(l,k),'k-',tvec);
        plot(t,y,'k-');
        [y,t] = step(tvec,stepsysHinf(l,k),'k--',tvec);
        plot(t,y,'k--');
    end
end
%changetext(gcf,'Step Response','Response to step demands');
xlabel('Time (s)')
ylabel('Amplitude')
axis([0 3 -.2 1.4])
legend('Our design','Optimal H^\infty design',0)

%
%   Plot the open-loop controllers
%   
figure(3)
sigma(outdata.sysKNPDC,'k-',xvec)
sigma(ltisys2sys(KHinf),'k--',xvec)

%
%   Get and display some data of the design
%
peakSNPDC = max(svSNPDC(:));
peakTNPDC = max(svTNPDC(:));
peakSHInf = max(svSHInf(:));
peakTHInf = max(svTHInf(:));

xvec2 = xvec;
while max(svSNPDC(:,1))<=1/sqrt(2)
    svSNPDC(:,1) = [];
    bandwidthSNPDC = xvec2(1);
    xvec2(1) = [];
end
xvec2 = xvec;
while max(svSHInf(:,1))<=1/sqrt(2)
    svSHInf(:,1) = [];
    bandwidthSHInf = xvec2(1);
    xvec2(1) = [];
end
xvec2 = xvec;
while max(svTNPDC(:,end))<=1/sqrt(2)
    svTNPDC(:,end) = [];
    bandwidthTNPDC = xvec2(end);
    xvec2(end) = [];
end
xvec2 = xvec;
while max(svTHInf(:,end))<=1/sqrt(2)
    svTHInf(:,end) = [];
    bandwidthTHInf = xvec2(end);
    xvec2(end) = [];
end

disp('  Peak S    Peak T   Bandwidth S  Bandwidth T  Order')
disp('NPDC')
disp([ peakSNPDC peakTNPDC bandwidthSNPDC bandwidthTNPDC ...
        size(outdata.sysKNPDC.a,1) ])
disp('HInf')
disp([ peakSHInf peakTHInf bandwidthSHInf bandwidthTHInf 17 ])

%
%   Printing
%
printflag = 1;
if printflag
    papersize = [ 15 7.5 ];
    width = 12;
    height = 5;
    left = (papersize(1)- width) / 2;
    bottom = (papersize(2)- height) / 2;
    myfiguresize = [left, bottom, width, height];
    papersize2 = [ 10 7.5 ];
    width2 = 7;
    height2 = 5;
    left2 = (papersize2(1)- width2) / 2;
    bottom2 = (papersize2(2)- height2) / 2;
    myfiguresize2 = [left2, bottom2, width2, height2];
    myfontsize = 12;
    myfontweight = 'bold';

    figure(1)
    set(gcf,'Units','centimeters')
    set(gcf, 'PaperPosition', myfiguresize);
    ChangeTextStyle(gcf,myfontsize,myfontweight);
    print sensitivityAIRC -deps
    
    figure(2)
    set(gcf,'Units','centimeters')
    set(gcf, 'PaperPosition', myfiguresize2);
    ChangeTextStyle(gcf,myfontsize,myfontweight);
    print stepAIRC -deps
    
end