%
%   This function solves the design example called "AIRC model" from [1], 
%    using the dgree-contrained Nevanlinna-Pick design procedure.
%
%   1. Maciejowski, J.M., "Multivariable Feedback Design", Addison-Wesley, 1989
%
function outdata = ExAIRCNPDC(indata);

%
%   Revive indata
%
gamma = indata.gamma;
kappa = indata.kappa;
speczeros = indata.speczeros;
maxdist = indata.maxdist;
lambda = indata.lambda;
addpoints = indata.addpoints;
poleposition = indata.poleposition;

%
% Load the model
%
load AIRCmodel

%
%   Perform the NPDC design
%

%
%   We modify by moving the pole in origin away from the imaginary axis
%
A(1,1) = poleposition; 

%
%   Stabilize and model matching
%
newpoles = -5:-1;
H = place(A',C(:,:)',newpoles)';
F = place(A,-B(:,:),newpoles);

AF = A+B*F;
AH = A-H*C;  % C2 = -C 
I = eye(size(D));

sysT1 = ss([AF, -B*F; zeros(size(A)) A-H*C],[zeros(size(B)); H],[-C-D*F D*F],I);
sysT2 = ss(AF,B,-C-D*F,-D);
sysT3 = ss(AH,H,-C,I);
sysT1 = minreal(sysT1);
sysT2 = minreal(sysT2);
sysT3 = minreal(sysT3);

%
%   Inner-outer factor T2 & T3. Let outer factors be absorbed by Q.
%
if isempty(zero(sysT2))
    sysT2in = ss([],[],[],I);
    sysT2out = sysT2;
else
    [sysT2in, sysT2inp, sysT2out] = iofr(sysT2);
    sysT2in = minreal(sysT2in);
    sysT2out = minreal(sysT2out);
end
[sysT3in, sysT3inp, sysT3out] = iofc(sysT3);
sysT3in = minreal(sysT3in);
sysT3out = minreal(sysT3out);

%
% Get interpolation data
%
svec = - [ pole(sysT2in); pole(sysT3in) ];
sysphi = zpk(svec,-svec,1);
sysT1hat = minreal(sysphi*inv(sysT2in)*sysT1*inv(sysT3in));

Ws = {};
for s = svec'
    Ws{end+1}{1} = sysT1hat.c*inv(eye(size(sysT1hat.a))*s-sysT1hat.a)*sysT1hat.b+sysT1hat.d;
end

%
%   Add interpolation conditions
%
svec = [  addpoints svec'  ];
Ws = [ {[{I}]} Ws ];

%
%   Get strictly proper controller, S''(inf) = 0
%
sysT2ininv = inv(sysT2in);
sysT3ininv = inv(sysT3in);
phim = GetMarkov(sysphi,3);
T2ininvm = GetMarkov(sysT2ininv,3);
T3ininvm = GetMarkov(sysT3ininv,3);
Sm(:,:,1) = I; Sm(:,:,2) = 0*I; Sm(:,:,3) = 0*I;
Shatm = MPolyMult(reshape(phim,1,1,3),reshape(T2ininvm,3,3,3));
Shatm = MatPolyMult(Shatm,Sm);
Shatm = MatPolyMult(Shatm,reshape(T3ininvm,3,3,3));

%
%   Transform from continuous time to standard form
%
domaintrans = GetBlTrans(kappa,-kappa,1,1);
rangetrans = GetBlTrans(1,gamma,-1,gamma);
scalingtrans = GetBlTrans(1,0,0,lambda);
s2ztrans = GetBlTrans(0,1,1,0);

interdatas = GetInterData(svec,Ws,speczeros,maxdist);
interdataz = BilinearDomainData(s2ztrans,interdatas);
W = interdataz.W;
W{1} = [ {Shatm(:,:,1)} {Shatm(:,:,2)} {Shatm(:,:,3)} ];
interdataz = GetInterData(interdataz.z,W,interdataz.speczeros,maxdist);
interdata1 = BilinearDomainData(scalingtrans,interdataz);
interdata2 = BilinearDomainData(domaintrans,interdata1);
interdata3 = BilinearRangeData(rangetrans,interdata2);

%
%   Solve NPDC
%
interpolant3 = NPDC(interdata3);
GetIntError(interpolant3,interdata3)

%
%   Transform back
%
interpolant2 = BilinearRange(rangetrans,interpolant3);
GetIntError(interpolant2,interdata2)

interpolant1 = BilinearDomain(domaintrans,interpolant2);
GetIntError(interpolant1,interdata1)

interpolants = BilinearDomain(scalingtrans,interpolant1);
GetIntError2(interpolants,interdatas)

%
%   Get controller
%
sysShat = ss(interpolants.A,interpolants.B,interpolants.C,interpolants.D);
sysS = minreal(inv(sysphi)*sysT2in*sysShat*sysT3in);

if norm(sysS.d - eye(3)) > 1e-8
    disp('Error: sysS.d should be identity'), norm(sysS.d - eye(3))
end

sysPs = ss(sys.a,sys.b,sys.c*sys.a,sys.c*sys.b);
sysSinv_I = inv(sysS);
if norm(sysSinv_I.c*sysSinv_I.b) > 1e-4
    disp('Error: syssSinv_I.d should be zero'), norm(sysSinv_I.c*sysSinv_I.b)
end
syssSinv_I = ss(sysSinv_I.a,sysSinv_I.b,sysSinv_I.c*sysSinv_I.a,0*I);%sysSinv_I.c*sysSinv_I.b);

[ z p k ] = zpkdata(syssSinv_I);
k(find(abs(k)<1e-5*norm(k))) = 0;
syssSinv_I = zpk(z,p,k);

tol = 1e-6;
sysKNPDC = minreal(Inv3by3(sysPs)*tf(syssSinv_I),tol);
sysKNPDC = minreal(ss(sysKNPDC));
Gsys = [ I -sys; I -sys ];
clsysNPDC = minreal(lft(Gsys,sysKNPDC));

%
%   Give outdata
%
outdata.sysKNPDC = sysKNPDC;
outdata.clsysNPDC = clsysNPDC;
outdata.sysShat = sysShat; 