%
%   This function solves the inverse multivariate Yule-Walker
%    equation.
%
%   Input:  A(:,:,k) = Ak, k = 1..n  (A0 need not be Id)
%           N - number of desired Markov coefficients R
%
%   Output: R(:,:,k) = Rk, k = 1..N  
%
%   where           A(z) = A0 + A1*z + ... + An*z^n
%         (A*(z)A(z))^-1 = R0 + R1(z+z^-1) + ...
%
function R = GetARCovariances(A,N)

% Some sizes
m = size(A,1);
n = size(A,3);
nm = n*m;

% Normalize
A0inv = inv(A(:,:,1));
for k = 1:n
    A(:,:,k) = A(:,:,k)*A0inv;
end

% Construct coefficient matrices
B = sparse(nm,nm);
C = sparse(nm,nm);
C(end-m+1:end,1:m) = A(:,:,n)';
for k = m:m:nm-1
    B(end-k+1:end-k+m,end-k+1:end) = ...
        [ B(end-k+1+m:end-k+2*m,end-k+m+1:end) A(:,:,k/m)' ];
    C(end-k+1-m:end-k,1:k+m) = [ A(:,:,n-k/m)' C(end-k+1:end-k+m,1:k) ];
end

% Construct permutation indices
hvec = [];
for k = 1:m
    hvec = [ hvec k:m:m^2 ];
end
vec = [];
for k = 0:m^2:nm*m-1
    vec = [vec hvec+k];
end

Im = speye(m);
B = kron(B',Im);
C = kron(C',Im);

% Permutation of B
B = B(:,vec);

% RHS
D = zeros(nm*m,1);
D(1:m^2) = Im(:);

% Solve system
Rvec = (B+C)\D;

% Extend up to N
R = reshape(Rvec,m,m,n);
Avec = reshape(A,m,nm);
Avec = Avec(:,m+1:end);
for k = n+1:N+1
    R(:,:,k) = - Avec * blktrans(R(:,:,k-1:-1:k-n+1));
end

% Reorder to output format
for k = 1:N+1   
    R(:,:,k) = full(A0inv*R(:,:,k)*A0inv');
end