%
%   This function solves the multivariate Yule-Walker
%    equation.
%
%   Input: A = { A0 A1 ... An }  (A0 need not be Id)
%
%   Output: R = { R0 R1 ... Rn }
%
%   where A(z) = A0 + A1*z + ... + An*z^n
%         (A*(z)A(z))^-1 = R0 + R1(z+z^-1) + ...
%
function R = MultivariateYuleWalker(A)

m = size(A{1},1)
n = length(A)
nm = n*m

% Normalize
A0 = A{1};
A0inv = inv(A0);
for k = 1:n
    A{k} = A0inv*A{k};
end

% Construct coefficient matrices

B = sparse(nm,nm);
C = sparse(nm,nm);
B(end-m+1:end,1:m) = A{n};
for k = m:m:nm-1
    B(end-k-m+1:end-k,1:k+m) = [ A{n-k/m} B(end-k+1:end-k+m,1:k) ];
    C(k+1:k+m,m+1:k+m) = [ A{k/m} C(k-m+1:k,m+1:k)];
    %B = [ B(1:m,:) A{n-k}; B zeros(size(B,1),m) ];
    %C = [ C zeros(size(C,1),m); A{k} C(end-m:end,:) ];
end

Im = speye(m);
B = kron(B,Im);
C = kron(C,Im);

% Construct permutation indices
vec1 = 1:nm:nm*m;
vec = [];
for k = 0:n-1
    vec = [vec vec1+k*m];
end
vec1 = vec;
vec = [];
for k = 0:m-1
    vec = [vec vec1+k];
end

% Permutation of C
C = C(:,vec);

% RHS
D = sparse(nm*m,1);
D(1:m^2) = Im(:);

% Solve system
Rvec = (B+C)\D;

% Reorder to output format
Rvec = reshape(Rvec,nm,m);
for k = 1:n
    R{k} = Rvec(k*m-m+1:k*m,:);
end