%
%   This function solves the degree constrained Nevanlinna-Pick 
%    interpolation problem for a desired set of spectral zeros.
%    Let m be the number of interpolation points taken with multi-
%    plicity minus one.
%
%   Input: interdata - an interpolation data set
%
%   Output: iterpolant - an interpolant data set
%
function interpolant = NPDC(interdata)

%
%   Parameters
%
maxiter = 50;           %   maximal number of main iiterations  
calcinterror = 0;       %   flag wether to determine interror or not
errorboundgrad = 1e-4;  %   maximal 2-norm of gradient (to be on trajectory) 
errorboundint = 1e-8;   %   maximal 2-norm of interpolation errors 
epsilon = 5e-2;         %   a number which determines predictor-step size
                        %   (ellipsoidal size)
slope = 3e-1;           %   slope bound in Wolfe test
                        %   large --> more inaccurate line search
digit = 8;              %   for display

%
%   Initialize
%
interror = 1;
graderror = 1;

%
%   Make data self-conjugated, move first interpolation point to zero
%    and check that data is valid.
%
interdataold = interdata;
[interdata, tozerotrans, symtrans] = DataValid(interdata);

%
%   Sizes etc
%
m = interdata.m;
mkn = interdata.mkn;
n = interdata.n;
A = diag(ones(mkn-2,1),1) + [zeros(mkn-2,mkn-1); -fliplr(interdata.tau(2:end)) ];
c = [ 1 zeros(1,mkn-2) ];
d = 1/2;

%
%   Determine Pick matrix
%
P = GetPickMatrix(interdata);
if min(eig(P))<=0,
  disp('Pick matrix is NOT positive!')
  eig(P)
  return
end
 
%
%   Generate L, V & K matrices
%
L = GetL(interdata);
LL = GetLL(interdata);
V = GetV(interdata);
invK = real(L*V*inv(P.')*V'*L');   % L is real so L' = L.'
K = inv(invK);
KK = kron(eye(m),K);
  
%
%   Determine central solution
%
invK11 = invK(1:m,1:m);     invK12 = invK(1:m,m+1:end);
invK21 = invK(m+1:end,1:m); invK22 = invK(m+1:end,m+1:end);
R0 = sqrtm(invK11.');
R1 = inv(R0)*invK21.';
R = reshape([R0 R1],m,m,mkn);

%
%   Determine the rho polynomial
%
if isempty(interdata.speczeros)
    rho = interdata.tau;
else
    rho = poly(interdata.speczeros);
    rho = [ zeros(1,length(interdata.tau)-length(rho)) rho ];
end


%
%   Check whether we are looking for the central solution
%
if norm(rho-interdata.tau) < eps
   disp('This is the central solution.')
   interpolant = GetInterpolant(R,rho);
   errorint = GetInterror(interpolant,interdata);
   if errorint > errorboundint,
       disp('Solution might be inaccurate'), error_int
   end
   interpolant = SvdFlip(interpolant,symtrans);
   interpolant = BilinearDomain(tozerotrans,interpolant);
   errorint = GetIntError(interpolant,interdataold);
   return
end

%
%   Determine the phi at the desired solution and normalization
%
psi = covar2(rho,interdata.tau,interdata.mkn-1);
phi = psi/psi(1);
rho = rho/sqrt(psi(1));

%
%   Hessian of g_rho at the central solution
%
muvec = Psi2Mu([phi(1) 0*phi(2:end)],interdata.tau);
h_f = GetHessian(R,muvec,interdata.tau,K);
if min(eig(h_f)) <= 0,
   disp('Hessian is not positive definite!')
   disp('Central solution might be inaccurate.')
   interpolant = GetInterpolant(R,interdata.tau);
   errorint = GetIntError(interpolant,interdata);
end

%
%   Calculation of eta * tau* + eta* * tau = muvec1
%
eta = [ 0; LL(1:end-1,1:end-1)*phi(2:end)' ];
muvec1 = (hankel(interdata.tau) + LL') * eta;

disp('                                                              ')
disp('--------------------------------------------------------------')
disp('Nevanlinna-Pick solver based on a continuation method         ')
disp('--------------------------------------------------------------')
disp('                    Corrector step                            ')
disp('     nu       iteration   gradient error interpolation error  ')
disp('--------------------------------------------------------------')

nu = 0;
while nu ~= 1

    %
    %   Predictor step:
    %    Calculate the direction for a step in nu
    %
    d_nu_grad_g = GetGradient(R,muvec1,interdata.tau,K) - 2*KK*R2VecRT(R);
    d_R_nu = - h_f \ d_nu_grad_g;

    %
    %   Determine the step size for nu
    %
    if norm(d_R_nu) == 0
        dnu = eps;
    else
        dnu = sqrt(epsilon/(d_R_nu'*KK*d_R_nu));
    end
    dnu = max(dnu,0.01);
    
    if nu + dnu > 1,
        dnu = 1 - nu;
    end
    
    muvec = Psi2Mu([phi(1) (nu+dnu)*phi(2:end)],interdata.tau);
 
    %
    %   We implicitly have to fulfill
    %       convex region, ie positive definite hessian
    %
    while min(eig(GetHessian(R + VecRT2R(d_R_nu*dnu,m,mkn), ...
                muvec,interdata.tau,K))) <= 0
        dnu = 0.5*dnu;
        muvec = Psi2Mu([phi(1) (nu+dnu)*phi(2:end)],interdata.tau);
    end
    nu = nu + dnu;
    disp(num2str(nu,digit))

    %
    %   Corrector step:
    %
    %       Update the polynomial alpha, generate beta and 
    %        check interpolation error.
    %
    R = R + VecRT2R(d_R_nu*dnu,m,mkn);

    iter = 0;
    int_error_old = 0;
    grad_error_old = 0;
    h_f = GetHessian(R,muvec,interdata.tau,K);
    g_f = GetGradient(R,muvec,interdata.tau,K);
    while (nu >= 1 | norm(g_f) > errorboundgrad) & ...
            (nu < 1 | GetIntError(GetInterpolant(R,rho),interdata) > errorboundint)
        d_R = - (h_f\g_f)';
 
        %
        %   One-sided Wolfe test. We also check that we
        %       stay in the convex region (pos def hessian)
        %   We try to take a full Newton step and use interval halfing
        %    until we fulfill the criteria above.
        %
        lambda = 1;
        g_f_new = GetGradient(R+VecRT2R(lambda*d_R',m,mkn),muvec,interdata.tau,K);
        h_f_new = GetHessian(R+VecRT2R(lambda*d_R',m,mkn),muvec,interdata.tau,K);
        
        pprim0 = d_R * g_f;
        pprimnew = d_R * g_f_new;
        pbisnew = d_R * h_f_new * d_R';
        
        itertmp = 0;
        while -slope*pprim0 < pprimnew | min(eig(h_f_new)) <= 0
             
            lambda = lambda / 2;
            g_f_new = GetGradient(R + VecRT2R(lambda*d_R,m,mkn),muvec,interdata.tau,K);
            h_f_new = GetHessian(R + VecRT2R(lambda*d_R,m,mkn),muvec,interdata.tau,K);
            pprimnew = d_R * g_f_new;
            
            itertmp = itertmp +1;
            if itertmp > 10
                break
            end
        end

        %
        %   Update the solution and check interpolation error
        %
        R = R + VecRT2R(lambda*d_R',m,mkn);
        h_f = h_f_new;
        g_f = g_f_new;

        %
        %   Calculate the error(s)
        %
        graderror =  norm(g_f);
        if nu == 1
            interror = GetIntError(GetInterpolant(R,rho),interdata);
        elseif calcinterror
            [ Asp Bsp Csp Dsp ] = SpecFactorization(A,nu*phi(2:end)',c,d);
            [ num den ] = ss2tf(Asp,Bsp,Csp,Dsp);
            interpolant = GetInterpolant(R,num);
            interror = GetIntError(interpolant,interdata);
        end
  
        iter = iter + 1;
        if mod(iter,1) == 0
            
            interrorold = interror; 
            graderrorold = graderror; 

            iterstr = num2str(iter);
            interrorstr = num2str(interror,digit);
            graderrorstr = num2str(graderror,digit);
            if calcinterror | nu == 1
                disp([blanks(20 - length(iterstr)) iterstr, ...
                        blanks(20 - length(graderrorstr)) graderrorstr, ...
                        blanks(20 - length(interrorstr)), interrorstr])
            else
                disp([blanks(20 - length(iterstr)) iterstr, ...
                        blanks(20 - length(graderrorstr)) graderrorstr])
            end
        end
        if iter > maxiter
            break
        end
    end
end

disp('--------------------------------------------------------------')
disp(' ')

interpolant = GetInterpolant(R,rho);
interpolant = SvdFlip(interpolant,symtrans);
interpolant = BilinearDomain(tozerotrans,interpolant);

disp('Finished')